__author__ = 'Martin'

class Address:
    def __init__(self, house_number, street, postcode):
        self.house_number = house_number
        self.street = street
        self.postcode = postcode

class Person:
    def __init__(self, given_name, family_name, other_name='',
                 house_number='', street='', postcode='',
                 email='', phone='', mobile=''):
        self.given_name = given_name
        self.other_name = other_name
        self.family_name = family_name
        self.address = Address(house_number, street, postcode)
        self.email = email
        self.phone = phone
        self.mobile = mobile

from array_types import array_
import os
import pickle
import copy


def _display_menu():
    print('\n   Options are:')
    print('      1) Display contact')
    print('      2) Add a new contact')
    print('      3) Update contact')
    print('      4) Display current contacts')
    print('      5) Delete contact')
    print('      0) Quit program')
    print()


def _get_book():
    book = array_(0)
    path = os.path.dirname(os.path.abspath(__file__))       # get path of address_book file
    file_path = os.path.join(path, 'book.data')             # allows for Windows \ in path
    if os.path.isfile(file_path):
        input_file = open('book.data', 'rb')
        try:
            while True:
                try:
                    rec = pickle.load(input_file)
                    book.append(rec)
                except (EOFError):
                    break
        finally:
            input_file.close()
        print('Contacts read in from ', file_path)
    else:
        print('Unable to find existing contacts file')
    return book, file_path


def _save_book(book):
    output_file = open('book.data', 'wb')
    try:
        for ix in range(book.size):
            pickle.dump(book[ix], output_file, pickle.HIGHEST_PROTOCOL)
    finally:
        output_file.close()


def _update_contact(contact):
    while True:
        temp = input('Given name ' + contact.given_name + ' >>> ')
        if temp != '': contact.given_name = temp
        temp = input('Family name ' + contact.family_name + ' >>> ')
        if temp != '': contact.family_name = temp
        temp = input('email ' + contact.email + ' >>> ')
        if temp != '': contact.email = temp
        temp = input('house_number ' + contact.address.house_number + ' >>> ')
        if temp != '': contact.address.house_number = temp
        temp = input('street ' + contact.address.street + ' >>> ')
        if temp != '': contact.address.street = temp
        temp = input('phone ' + contact.phone + ' >>> ')
        if temp != '': contact.phone = temp
        temp = input('mobile ' + contact.mobile + ' >>> ')
        if temp != '': contact.mobile = temp
        option = input("Details OK? Y/N 'C' to cancel >>> ")
        if option in ['y', 'Y', 'c', 'C']:
            break
    if option in ['y', 'Y']:
        return contact
    else:
        return None


def _get_contact(book):
    gname = input('Given name: >>> ')
    fname = input('Family name: >>> ')
    found = False
    ix = 0
    while ix < book.size:
        person = book[ix]
        if person.family_name == fname and person.given_name == gname:
            found = True
            break
        ix += 1
    if found:
        return person, ix
    else:
        return None, ix


def _display_contact(person):
    print(person.given_name, person.family_name)
    print(person.address.house_number, person.address.street, person.address.postcode)
    print('email: ', person.email, '  phone: ', person.phone, '  mobile: ', person.mobile)
    print('------------------------------------------------------------------------------')


def _display_contacts(book):
    print('------------------------------------------------------------------------------')
    for person in book:
        _display_contact(person)


def _show_contact(book):
    print('\nEnter existing contact name:')
    rec, ix = _get_contact(book)
    print('------------------------------------------------------------------------------')
    if rec:
        _display_contact(rec)
    else:
        print('No such contact')


def _new_contact(book):
    print('\nEnter new contact details: return to skip a field')
    new_person = _update_contact(Person('',''))
    if new_person:
        book.append(new_person)
        return True
    else:
        print('New contact canceled')
        return False


def _update_details(book):
    print('\nEnter contact to update:')
    person, ix = _get_contact(book)
    if person:
        _display_contact(person)
        contact = copy.copy(person)        # shallow copy but address is a field
        contact.address = copy.copy(contact.address)
        updated = _update_contact(contact)
        if updated:
            book[ix] = updated
            return True
        else:
            print('Update canceled')
            return False
    else:
        print('No such contact')
        return False


def _delete_contact(book):
    print('\nEnter contact to delete:')
    person, ix = _get_contact(book)
    if person:
        _display_contact(person)
        print('Are you sure you want to delete this person???')
        option = input('Y/N >>> ')
        if option in ('Y', 'y'):
            book.pop(ix)
            book.size -= 1
            return True
        else:
            print('Deletion canceled')
            return False
    else:
        print('No such contact')
        return False


def address_book():
    print('\nWelcome to ADDRESS BOOK')
    updated = False
    book, file_path = _get_book()

    while True:
        _display_menu()
        option = input('please input an option >>> ')
        if option == '0': break
        if option == '1':
            _show_contact(book)
        elif option == '2':
            updated = _new_contact(book) or updated      # note that new_contact() must come first in or
        elif option == '3':
            updated = _update_details(book) or updated
        elif option == '4':
            _display_contacts(book)
        elif option == '5':
            updated = _delete_contact(book) or updated

    print('\nThank you for using ADDRESS BOOK')
    if updated:
        _save_book(book)
        print('saved to', file_path)

address_book()



# a_person = Person('Jonathan', 'Hansen')
# a_person.email = 'jhansen@whatercom.com'
# a_person.address.house_number = 22
# a_person.address.street= 'Hurdle Lee'
#
# print(a_person.given_name, a_person.family_name, a_person.address.house_number,
#       a_person.address.street, a_person.email)
#
# b_person = copy.deepcopy(a_person)
#
# a_person.family_name = 'Smith'
#
# print(b_person.given_name, b_person.family_name)